#ifndef SPY_H
#define SPY_H

#include <vector>
#include <map>
#include <fstream>
#include <qobject.h>
#include <qtimer.h>
#include "kadu.h"
#include "spy_slots.h"
#include "spy_list.h"


class Spy : public QObject
{
    Q_OBJECT
		QTimer *timerConnected, *timerCheck;
	
		// Lista osób obecnie ukrytych, sprawdzanych czy nie stali się niedostępni.
		std::vector<QString> invisible;
		
		// Lista osób, które wtyczka ma sprawdzać.
		std::vector<QString> tracked;

		// Lista osób, którym wysłano pakiet i czekamy na odpowiedź.
		std::vector<QString> sent;
	
		SpySlots *spy_slots;
	
		/* Zmienia określa czy minęło < 10 sek. od połączenia z serwerem.
		 * Jeżeli jest prawdziwa, to ignorujemy zmiany statusu na niedostępny
		 * oraz żadania wysłania obrazka, bo są to wiadomości zakolejkowane z serwera.
		 */
		bool rightAfterConnection;
	
		int checkInvisibleStatus (QString ID, bool forceCheck = false);
		void unsetInvisible ();
		void createGUI ();	
		void destroyGUI ();
		void loadTrackedList ();
		void saveTrackedList ();
		bool isTracked (const QString& uin);
		bool isOnTrackedList (const QString& uin);
		bool isChattingWith (const UserGroup& users);
		bool isChattingWith (const QString& uin);
		void notifyAboutScanning(const QString& uin);
		bool isOnSentList(const QString& uin);
		
		SpyList *spy_list;
	private slots:
		void scanUser();
		void checkNow();
	
	public:
		Spy();
		~Spy();

		void trackedRemove (const QString& uin);
		void trackedAdd (const QString& uin);
	public slots:
		void imageRequestReceived (UinType sender, uint32_t size, uint32_t crc32);
		void statusChanged(UserListElement user, QString protocolName, const UserStatus &oldstatus, bool massively, bool last);
		void connected ();
		void check ();
		void clearInvisible ();
		void messageFiltering (Protocol *protocol, UserListElements senders, QCString& msg, QByteArray& formats, bool& stop);
		void popupMenu ();
		void newMessage (Protocol *protocol, UserListElements senders, const QString &msg, time_t t);

	friend class SpySlots;
};

#endif
